#| auto-stop-processes.jl -- stop and continue a window's processes

   Version 0.1

   Copyright (C) 2002 Daniel Pfeiffer <occitan@esperanto.org>

   This file is part of sawfish.

   sawfish is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   sawfish is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with sawfish; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
|#

(define-structure sawfish.wm.state.auto-stop-processes

    (export toggle-window-auto-stopable
	    get-pids)

    (open rep
	  rep.system
	  rep.io.processes
	  sawfish.wm.commands
	  sawfish.wm.custom
	  sawfish.wm.menus
	  sawfish.wm.misc
	  sawfish.wm.windows
	  sawfish.wm.workspace
	  sawfish.wm.ext.match-window)

  (defcustom auto-stop-on-other-viewports ()
    "Temporarily completely stop auto-stoppable windows appearing only on other viewports."
    :type boolean
    :group misc
    :require sawfish.wm.viewport
    :tooltip "This only takes effect when restarting sawfish.")

  (defcustom auto-stop-on-other-workspaces t
    "Temporarily completely stop auto-stoppable windows appearing only on other workspaces."
    :type boolean
    :group misc
    :tooltip "This only takes effect when restarting sawfish.")

  (defvar auto-stop-ps-command
    `(0 "ps" "-u" ,(or (getenv "USER") (getenv "LOGNAME")))
    "Command called for finding pids when window property is a regexp.
First element is the column number where to read the pid.  Rest are the
command and arguments.")

  (define auto-stops ())

  (define (get-pids w)
    (let ((pids (window-get w 'pids)))
      (if (and (consp pids) (numberp (car pids)))
	  pids
	(if (stringp pids)
	    (let* ((stream (make-string-output-stream))
		   (proc (make-process stream))
		   l)
	      (require 'rep.regexp)
	      (set-process-error-stream proc standard-error)
	      (apply call-process proc nil (cdr auto-stop-ps-command))
	      (map-windows (lambda (w)
			     (and (setq pids (window-get w 'pids))
				  (stringp pids)
				  (setq l `((,w ,pids) ,@l)))))
	      (setq stream (make-string-input-stream (car stream)))
	      (while (setq proc (read-line stream))
		(setq proc (substring proc 0 (1- (length proc))))
		(mapc (lambda (w)
			(if (string-match (cadr w) proc)
			    (setcdr (cdr w)
				    `(,(read-from-string
					(substring proc (car auto-stop-ps-command)))
				      ,@(cddr w)))))
		      l))
	      (mapc (lambda (w)
		      (window-put (car w) 'pids (cddr w)))
		    l)
	      (cddr (assq w l)))
	  (window-put
	   w 'pids
	   (cond
	    ((functionp pids)
	     (funcall pids w))
	    ((setq pids (get-x-property w '_NET_WM_PID))
	     (vector->list (caddr pids)))
	    ((require 'sawfish.wm.util.prompt)
	     (let ((pid (read-from-string
			 (prompt (concat (window-name w)
					 "\n\n(Hit return to abort)\nProcess ID(s): ")))))
	       (if (consp pid) pid (if pid `(,pid)))))))))))

  (define-macro (cont when cond)
    `(setq auto-stops
	   (let (continued)
	     (delete-if
	      (lambda (c)
		(if (not (window-id (car c)))
		    t
		  (if ,cond
		      (let* ((cont (and ,when t))
			     (sig (if cont 'CONT 'STOP)))
			(mapc (lambda (pid)
				(unless (memq pid continued)
				  (if cont
				      (setq continued `(,pid ,@continued)))
				  (signal-process pid sig)))
			      (get-pids (car c)))
			(setcdr c (not cont))))
		  ()))
	      auto-stops))))

  (define (auto-stop-by-viewport)
    (require 'sawfish.wm.viewport)
    (cont (or (window-get (car c) 'sticky-viewport)
	      (not (window-outside-viewport-p (car c))))
	  (window-appears-in-workspace-p (car c) current-workspace)))

  (define (auto-stop-by-workspace #!rest args)
    (declare (unused args))
    (cont (window-appears-in-workspace-p (car c) current-workspace)
	  t))

  (when auto-stop-on-other-viewports
    (add-hook 'viewport-moved-hook auto-stop-by-viewport))
  (when auto-stop-on-other-workspaces
    (add-hook 'enter-workspace-hook auto-stop-by-workspace))


  (define (auto-stop-continue-all)
    (cont (cdr c) t))

  (add-hook 'before-exit-hook auto-stop-continue-all)


  (define (toggle-window-auto-stopable w)
    (if (assq w auto-stops)
	(setq auto-stops (delq (assq w auto-stops) auto-stops))
      (if (get-pids w)
	  (setq auto-stops `((,w) ,@auto-stops)))))

  (define-command 'toggle-window-auto-stopable toggle-window-auto-stopable #:spec "%W")

  (add-window-menu-toggle (_ "Processes Auto-_stopable")
			  'toggle-window-auto-stopable
			  (lambda (w) (if (assq w auto-stops) t)))


  (define (auto-stop-processes w prop value)
    (declare (unused prop))
    (when value
      (setq auto-stops `((,w) ,@auto-stops))))

  (define-match-window-property 'auto-stop-processes 'state 'boolean)

  (define-match-window-setter 'auto-stop-processes auto-stop-processes)

  (define-match-window-property 'pids 'state 'string))
